"""Collection of tests to ensure correct implementation and track progress. """

from fastapi.testclient import TestClient
import pytest

from .test_utils import landmarks_to_osmid, load_trip_landmarks, log_trip_details
from ..main import app

@pytest.fixture(scope="module")
def client():
    """Client used to call the app."""
    return TestClient(app)


def test_turckheim(client, request):    # pylint: disable=redefined-outer-name
    """
    Test n°1 : Custom test in Turckheim to ensure small villages are also supported.

    Args:
        client:
        request:
    """
    duration_minutes = 15
    response = client.post(
        "/trip/new",
        json={
            "preferences": {"sightseeing": {"type": "sightseeing", "score": 5},
            "nature": {"type": "nature", "score": 5},
            "shopping": {"type": "shopping", "score": 5},
            "max_time_minute": duration_minutes,
            "detour_tolerance_minute": 0},
            "start": [48.084588, 7.280405]
            }
        )
    result = response.json()
    landmarks = load_trip_landmarks(client, result['first_landmark_uuid'])

    # Add details to report
    log_trip_details(request, landmarks, result['total_time'], duration_minutes)

    # checks :
    assert response.status_code == 200  # check for successful planning
    assert isinstance(landmarks, list)  # check that the return type is a list
    assert duration_minutes*0.8 < int(result['total_time']) < duration_minutes*1.2
    assert len(landmarks) > 2           # check that there is something to visit


def test_bellecour(client, request) :   # pylint: disable=redefined-outer-name
    """
    Test n°2 : Custom test in Lyon centre to ensure proper decision making in crowded area.
    
    Args:
        client:
        request:
    """
    duration_minutes = 30
    response = client.post(
        "/trip/new",
        json={
            "preferences": {"sightseeing": {"type": "sightseeing", "score": 5},
                            "nature": {"type": "nature", "score": 5},
                            "shopping": {"type": "shopping", "score": 5},
                            "max_time_minute": duration_minutes,
                            "detour_tolerance_minute": 0},
            "start": [45.7576485, 4.8330241]
            }
        )
    result = response.json()
    landmarks = load_trip_landmarks(client, result['first_landmark_uuid'])
    osm_ids = landmarks_to_osmid(landmarks)

    # Add details to report
    log_trip_details(request, landmarks, result['total_time'], duration_minutes)

    # checks :
    assert response.status_code == 200  # check for successful planning
    assert duration_minutes*0.8 < int(result['total_time']) < duration_minutes*1.2
    assert 136200148 in osm_ids         # check for Cathédrale St. Jean in trip


def test_shopping(client, request) :   # pylint: disable=redefined-outer-name
    """
    Test n°3 : Custom test in Lyon centre to ensure shopping clusters are found.
    
    Args:
        client:
        request:
    """
    duration_minutes = 600
    response = client.post(
        "/trip/new",
        json={
            "preferences": {"sightseeing": {"type": "sightseeing", "score": 0},
                            "nature": {"type": "nature", "score": 0},
                            "shopping": {"type": "shopping", "score": 5},
                            "max_time_minute": duration_minutes,
                            "detour_tolerance_minute": 0},
            "start": [45.7576485, 4.8330241]
            }
        )
    result = response.json()
    landmarks = load_trip_landmarks(client, result['first_landmark_uuid'])
    osm_ids = landmarks_to_osmid(landmarks)

    # Add details to report
    log_trip_details(request, landmarks, result['total_time'], duration_minutes)

    # checks :
    assert response.status_code == 200  # check for successful planning
    assert duration_minutes*0.8 < int(result['total_time']) < duration_minutes*1.2

# def test_new_trip_single_prefs(client):
#     response = client.post(
#         "/trip/new",
#         json={
#             "preferences": {"sightseeing": {"type": "sightseeing", "score": 1},
#             "nature": {"type": "nature", "score": 1},
#             "shopping": {"type": "shopping", "score": 1},
#             "max_time_minute": 360,
#             "detour_tolerance_minute": 0},
#             "start": [48.8566, 2.3522]
#             }
#         )
#     assert response.status_code == 200


# def test_new_trip_matches_prefs(client):
#     pass
